local L = LibStub("AceLocale-3.0"):GetLocale("Chinon")
-- Un bouton servant aux soins

local APortee_Margin = 2
local BarreVie_Margin = 2

function SoinBouton_SetTexte(self, pos)
	self.prio:SetText(pos);
end

function SoinBouton_AddBuff(self, icon, count, expirationTime)
	self.nbBuffs=self.nbBuffs+1
	self.buff[self.nbBuffs]:Show();
	self.buff[self.nbBuffs].texture:SetTexture(icon);
	local remains = expirationTime-GetTime()
	if (count==0) then
		count = math.floor(remains)
	end
	self.buff[self.nbBuffs].stacks:SetText(count);
	if (remains<1.5) then
		self.buff[self.nbBuffs].stacks:SetTextColor(1,0,0);
	elseif (remains<=3) then
		self.buff[self.nbBuffs].stacks:SetTextColor(1,0.5,0);
	else
		self.buff[self.nbBuffs].stacks:SetTextColor(1,1,1);
	end
end

function SoinBouton_OnUpdate(self)
	local unit=self:GetAttribute("unit");
	local cfgBouton = self.cfgBouton;
	
	if (cfgBouton == nil) then
		return;
	end
	
	if (not UnitExists(unit) or UnitIsDeadOrGhost(unit)) then
		self.vie:Hide();
		self.viePerdue:Hide()
		self.barreSoin:Hide();
		self.barreDegats:Hide();
		for i,aPortee in pairs(self.aPortee) do
			aPortee:Hide();
		end
		
		for i,debuff in pairs(self.debuff) do		
			debuff:Hide();
		end
--		self.prio:Hide();
		return;
	end
	
	
	if (cfgBouton.vie and UnitHealthMax(unit) and UnitHealthMax(unit)>0) then
		self.vie:Show();
		self.viePerdue:Show()
		local rapport=UnitHealth(unit)/UnitHealthMax(unit);
		if (rapport>0.9) then
			self.vie:SetGradient("HORIZONTAL", 0,0.8,0, 0,0.5,0)
		elseif (rapport>0.5) then
			self.vie:SetGradient("HORIZONTAL", 0.5,1,0, 0.5,0.75,0);
		elseif (rapport>0.1) then
			self.vie:SetGradient("HORIZONTAL", 0.65,0.2,0, 0.45,0,0);
		else
			self.vie:SetGradient("HORIZONTAL", 1,0,0, 0.8,0,0);
		end
		if (rapport>1) then
			rapport=1;
		end
		self.vie:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2));
		if (rapport<1) then
			self.viePerdue:SetWidth((1-rapport)*(self:GetWidth()-BarreVie_Margin*2));
		else
			self.viePerdue:Hide()
		end

		if (self.soin>0) then
			local rapport = self.soin/UnitHealthMax(unit)
			if (rapport>1) then
				rapport=1
			end
			
			self.barreSoin:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2))
			self.barreSoin:Show()
		else
			self.barreSoin:Hide();
		end
		
		if (SoinFrame.totalDegats[unit] and SoinFrame.totalDegats[unit]>0) then
			local rapport = SoinFrame.totalDegats[unit]/UnitHealthMax(unit)
			if (rapport>1) then
				rapport=1
			end
			self.barreDegats:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2))
			self.barreDegats:Show()
		else
			self.barreDegats:Hide();
		end
	else
		self.viePerdue:Hide()
		self.vie:Hide();
		self.barreSoin:Hide();
		self.barreDegats:Hide()
	end

	if (cfgBouton.aggro) then
		if (UnitThreatSituation(unit)>1) then
			self:SetBackdropBorderColor(1,0,0);
			--self.aggro:Show();
		else
			--self.aggro:Hide()
			self:SetBackdropBorderColor(1,1,1);
		end
	else
		--self.aggro:Hide();
	end

	-- portée
	
	for i, portee in pairs(self.aPortee) do
		portee:Hide();
	end
		
	for i, portee in pairs(cfgBouton.portee) do
		if (portee.sort and IsSpellInRange(GetSpellInfo(portee.sort),unit)==1) then
			self.aPortee[i]:Show();
		end
	end
	
	-- Buffs
	for i in ipairs(cfgBouton.buff) do
		self.buff[i]:Hide()
	end
	
	self.nbBuffs=0
	
	if (#cfgBouton.buff) then
		local i=1;
		while 1 do
			local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable =  UnitBuff(unit, i, true);
			if (not name) then
				break;
			end
			if (cfgBouton.sortBuff[name] and (isMine or cfgBouton.sortBuff[name].tous)) then
				self:AddBuff(icon, count, expirationTime)
			end
			i=i+1;
		end
	end
	

	-- Debuffs	
	if (#cfgBouton.debuff) then
		for i, debuff in pairs(cfgBouton.debuff) do
			self.isDebuff[i] = false;
			self.debufLevel[i] = 0;
		end
		
		local i=1;
		while 1 do
			local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType, duration, expirationTime =UnitDebuff(unit,i);
			if (not debuffName) then
				break;
			end
			
			if (cfgBouton.sortDebuff2[debuffName]~= nil) then
				self:AddBuff(debuffTexture, debuffApplications, expirationTime)
			end
			
			local typeDebuff = 0;
			
			if (cfgBouton.sortDebuff[debuffName]~= nil) then
				typeDebuff = cfgBouton.sortDebuff[debuffName];
				self.isDebuff[typeDebuff] = true;
			end
				
			if (cfgBouton.sortDebuff[debuffType]~= nil) then
				typeDebuff = cfgBouton.sortDebuff[debuffType];
				self.isDebuff[typeDebuff] = true;
			end

			if (typeDebuff>0) then
				if (Chinon.decurseLevel[debuffName] and Chinon.decurseLevel[debuffName]>self.debufLevel[typeDebuff]) then
					self.debufLevel[typeDebuff] = Chinon.decurseLevel[debuffName];
				end

				if (Chinon.decurseLevel[debuffType] and Chinon.decurseLevel[debuffType]>self.debufLevel[typeDebuff]) then
					self.debufLevel[typeDebuff] = Chinon.decurseLevel[debuffType];
				end
			end
			
			i=i+1;
		end

		for i, debuff in pairs(self.debuff) do
			debuff:Hide()
		end
		
		for i, debuff in pairs(cfgBouton.debuff) do
			if (self.isDebuff[i]) then
				if (cfgBouton.porteeDebuff[i] and IsSpellInRange(cfgBouton.porteeDebuff[i],unit)==1) then
					if (self.debufLevel[i]==2) then
						self.debuff[i]:SetTexture(1.0,0.0,0.0);
					elseif (self.debufLevel[i]==1) then
						self.debuff[i]:SetTexture(1.0,1.0,0.0);
					else
						self.debuff[i]:SetTexture(0.2,0.2,1.0);
					end
				else
					self.debuff[i]:SetTexture(0,0,0);
				end
				self.debuff[i]:Show();
			end
		end
	end
end

function SoinBouton_OnLoad(self)
end

function SoinBouton_Init(self)
	self.Configure = SoinBouton_Configure;
	self.OnUpdate = SoinBouton_OnUpdate;
	self.SetTexte = SoinBouton_SetTexte;
	self.AddSpell = SoinBouton_AddSort;
	self.OnEnter = SoinBouton_OnEnter;
	self.OnLeave = SoinBouton_OnLeave;
	self.AddBuff = SoinBouton_AddBuff
	self.isDebuff = {}
	self.debufLevel = {}
end

function SoinBouton_Configure(self, cfgBouton)
	self.cfgBouton = cfgBouton;
    
	Chinon:AppliquerFrameSkin(self, Chinon.apparence.buttonSkin)
	self:SetBackdropBorderColor(1,1,1);
	
	self:SetWidth(Chinon.apparence.largeurBouton);
	self:SetHeight(Chinon.apparence.hauteurBouton);
	self:RegisterForClicks("AnyUp");

	--La vie restante
	if (not self.vie) then
		self.vie = self:CreateTexture();
		self.vie:SetTexture(1,1,1);
		self.vie:SetDrawLayer("ARTWORK");
	end
	self.vie:SetPoint("TOPLEFT",BarreVie_Margin,-BarreVie_Margin);
	self.vie:SetWidth(self:GetWidth()-BarreVie_Margin*2)
	self.vie:SetHeight(self:GetHeight()/2-BarreVie_Margin);
	
	--La vie perdue (permet de compléter la barre de vie)
	if (not self.viePerdue) then
		self.viePerdue = self:CreateTexture();
		self.viePerdue:SetTexture(0,0,0,0.5);
		self.viePerdue:SetDrawLayer("ARTWORK");
	end
	self.viePerdue:SetPoint("TOPRIGHT",-BarreVie_Margin,-BarreVie_Margin);
	self.viePerdue:SetWidth(0)
	self.viePerdue:SetHeight(self:GetHeight()/2-BarreVie_Margin);
	
	--Créer la barre des soins reçus
	if (not self.barreSoin) then
		self.barreSoin = self:CreateTexture();
		self.barreSoin:SetDrawLayer("OVERLAY")
		self.barreSoin:SetPoint("TOPRIGHT",-BarreVie_Margin,-BarreVie_Margin)
		self.barreSoin:SetTexture(0,0.2,0.8)
	end
	self.barreSoin:SetWidth(0)
	self.barreSoin:SetHeight(self:GetHeight()/4-BarreVie_Margin/2)
	self.soin = 0

	--Créer la barre des dégâts reçus
	if (not self.barreDegats) then
		self.barreDegats = self:CreateTexture();
		self.barreDegats:SetDrawLayer("OVERLAY")
		self.barreDegats:SetTexture(1,0,0)
	end
	self.barreDegats:SetPoint("TOPLEFT",-BarreVie_Margin,-self:GetHeight()/4-BarreVie_Margin)
	self.barreDegats:SetWidth(0)
	self.barreDegats:SetHeight(self:GetHeight()/6)
		
	--Créer les indicateurs de portée
	if (not self.aPortee) then
		self.aPortee = {}
	end		
	
	for i, portee in ipairs(cfgBouton.portee) do
		if (not self.aPortee[i]) then
			self.aPortee[i] = self:CreateTexture();
		end
		self.aPortee[i]:SetTexture("Interface\\AddOns\\Chinon\\Texture\\Portee.tga")
		self.aPortee[i]:SetPoint("BOTTOMLEFT",APortee_Margin+self:GetWidth()/6*(i-1),APortee_Margin);
		self.aPortee[i]:SetHeight(self:GetHeight()/3-APortee_Margin*2);
		self.aPortee[i]:SetWidth(self:GetWidth()/6-APortee_Margin*2);
	end
	
	--Créer zone de debuff en arrière plan du bouton
	if (not self.debuff) then
		self.debuff = {}
	end
	
	local largeurDebuff = (Chinon.apparence.largeurBouton-BarreVie_Margin*2) / #cfgBouton.debuff;
	for i, debuff in ipairs(cfgBouton.debuff) do
		if (not self.debuff[i]) then
			self.debuff[i] = self:CreateTexture();
		end
		self.debuff[i]:SetDrawLayer("BORDER");
		self.debuff[i]:SetPoint("TOPLEFT",largeurDebuff*(i-1)+BarreVie_Margin,-BarreVie_Margin);
		self.debuff[i]:SetHeight(self:GetHeight()-BarreVie_Margin*2);
		self.debuff[i]:SetWidth(largeurDebuff);
		self.debuff[i]:SetTexture(0,0,0);
	end

	--Le nom de la cible (en raid uniquement)
	if (cfgBouton.nomCible and GetNumRaidMembers()>5) then
		if (not self.prio) then
			self.prio = self:CreateFontString(nil, "OVERLAY");
		end
		self.prio:SetFontObject("GameFontNormal");
		self.prio:SetAllPoints(self);
		self.prio:SetText(UnitName(self:GetAttribute("unit")));
		self.prio:SetTextColor(1,1,1);
		self.prio:Show()
	else
		if (self.prio) then
			self.prio:Hide()
		end
	end
	
	--Les icônes de buff/debuff
	if (not self.buff) then
		self.buff = {}
	end
	
	local largeurBuff = Chinon.apparence.hauteurBouton / 2
		
	local nombreBoutonsBuff = #cfgBouton.buff + #cfgBouton.debuff2
	
	for i=1,nombreBoutonsBuff do
		if (not self.buff[i]) then
			self.buff[i] = CreateFrame("Frame",nil,self,nil)
			self.buff[i].texture = self.buff[i]:CreateTexture()
			self.buff[i].stacks = self.buff[i]:CreateFontString(nil, "OVERLAY");
			self.buff[i].stacks:SetNonSpaceWrap(false)
		end
		
		self.buff[i]:SetPoint("BOTTOMRIGHT",-largeurBuff*(i-1),0);
		self.buff[i]:SetHeight(largeurBuff);
		self.buff[i]:SetWidth(largeurBuff);
		self.buff[i].texture:SetAllPoints(self.buff[i])
		
		self.buff[i].stacks:SetFontObject("GameFontNormal");
		self.buff[i].stacks:SetAllPoints(self.buff[i])
		self.buff[i].stacks:SetText("");
		self.buff[i].stacks:SetTextColor(1,1,1);
		self.buff[i]:Hide();
	end
end

function SoinBouton_AddSort(self, sort, cible)
	
	if (sort~=nil and (sort.sort~=nil or sort.macro~=nil)) then
		local modifieur = "";
		if (sort.alt) then
			modifieur = modifieur.."alt-"
		end
		if (sort.ctrl) then
			modifieur = modifieur.."ctrl-"
		end
		if (sort.shift) then
			modifieur = modifieur.."shift-"
		end
		self:SetAttribute(modifieur.."type"..sort.bouton,"macro");
		local macro;
		if (sort.macro and sort.macro~="") then
			macro = sort.macro
		else
			local nom = GetSpellInfo(sort.sort);
			if (nom == nil) then
				DEFAULT_CHAT_FRAME:AddMessage("Impossible de trouver le sort "..sort.sort);
			end
			if (sort.rangInferieur and sort.rang) then
				nom = nom .. "(" .. sort.rang .. ")"		
			end
			macro = "/cast [target="..cible.."] "..nom;
		end
		self:SetAttribute(modifieur.."macrotext"..sort.bouton,macro);
	end
end

function SoinBouton_OnEnter(self)
	local unit = self:GetAttribute("unit")
	if (UnitName(unit)) then
		ChinonDebuffs:SetUnit(unit)
	end
end

function SoinBouton_OnLeave(self)
	ChinonDebuffs:Hide()
end

